@@ -248,7 +248,7 @@ class SupplyCurve:
             trans_table.rename(columns={'trans_line_gid': 'trans_gid',
                                         'trans_gids': 'trans_line_gids'})
 
-        if 'dist_mi' in trans_table:
+        if 'dist_mi' in trans_table and 'dist_km' not in trans_table:
             trans_table = trans_table.rename(columns={'dist_mi': 'dist_km'})
             trans_table['dist_km'] *= 1.60934
 
@@ -1020,7 +1020,7 @@ class SupplyCurve:
     def simple(cls, sc_points, trans_table, fcr, sc_features=None,
                transmission_costs=None, sort_on='total_lcoe',
                columns=('trans_gid', 'trans_type', 'lcot', 'total_lcoe',
-                        'trans_cap_cost_per_mw'),
+                        'dist_km', 'trans_cap_cost_per_mw'),
                max_workers=None, wind_dirs=None, n_dirs=2, downwind=False,
                offshore_compete=False):
         """
